/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2009, Tomasz Sowa
 * All rights reserved.
 * 
 */

#include "mount.h"




Mount::Mount()
{
	type   = cms;
	dir_id = -1;
	fs     = simplefs;

	param.resize(par_none);
}



const char * Mount::TypeToStr()
{
static char buffer[30];
	
	switch( type )
	{
		case cms:
		sprintf(buffer, "cms");
		break;	
	
		case thread:
		sprintf(buffer, "thread");
		break;
		
		case ticket:
		sprintf(buffer, "ticket");
		break;

		default:
		sprintf(buffer, "unknown");
		break;	
	}

return buffer;
}


const char * Mount::FsToStr()
{
static char buffer[30];
	
	switch( fs )
	{
		case simplefs:
		sprintf(buffer, "simplefs");
		break;	
	
		case hashfs:
		sprintf(buffer, "hashfs");
		break;
		
		default:
		sprintf(buffer, "unknown");
		break;	
	}

return buffer;
}



Mount::ParamCode Mount::ParseParam(const char * param_name)
{
	struct ParName
	{
		ParamCode param_code;
		const char * name;
	};

	static ParName par_name_tab[] =	{
		{ par_page,						"page" },
		{ par_thread,					"thread" },
		{ par_ticket,					"ticket" },
		{ par_ticket_type,				"ticket_type" },
		{ par_ticket_type_default,		"ticket_type_default" },
		{ par_ticket_status,			"ticket_status" },
		{ par_ticket_status_default,	"ticket_status_default" },
		{ par_ticket_priority,			"ticket_priority" },
		{ par_ticket_priority_default,	"ticket_priority_default" },
		{ par_ticket_category,			"ticket_category" },
		{ par_ticket_category_default,	"ticket_category_default" },
		{ par_ticket_expected,			"ticket_expected" },
		{ par_ticket_expected_default,	"ticket_expected_default" },
		{ par_createthread_on,			"createthread_on" },
		{ par_createticket_on,			"createticket_on" },
		{ par_only_root_remove,			"only_root_remove" },
		{ par_emacs_on,					"emacs_on" },
		{ par_mkdir_on,					"mkdir_on" },
		{ par_app,						"app" }
	};

	size_t i, len = sizeof(par_name_tab) / sizeof(ParName);

	for(i=0 ; i<len ; ++i)
	{
		if( strcmp(par_name_tab[i].name, param_name) == 0 )
			return par_name_tab[i].param_code;
	}

return par_none;
}



void Mount::ClearParams()
{
size_t i;

	for(i=0 ; i<param.size() ; ++i)
		param[i].Clear();
}


bool Mount::IsPar(Mount::ParamCode code)
{
	if( !param[code].defined )
		return false;

return true;
}



bool Mount::IsArg(Mount::ParamCode code, const char * arg)
{
ParamArg::iterator i;	

	if( !param[code].defined )
		return false;

	for(i=param[code].arg.begin() ; i!=param[code].arg.end() ; ++i)
	{
		if( *i == arg )
			return true;
	}

return false;
}


bool Mount::IsArg(Mount::ParamCode code, const std::string & arg)
{
	return IsArg(code, arg.c_str());
}


bool Mount::IsArg(Mount::ParamCode code, int arg)
{
ParamArg::iterator i;	

	if( !param[code].defined )
		return false;

	for(i=param[code].arg.begin() ; i!=param[code].arg.end() ; ++i)
	{
		if( atoi(i->c_str()) == arg )
			return true;
	}

return false;
}


